@@ -61,21 +61,32 @@ func (l *logConsumer) Register(name string) {
 }
 
 func (l *logConsumer) register(name string) *presenter {
-	cf := monochrome
-	if l.color {
-		if name == api.WatchLogger {
-			cf = makeColorFunc("92")
-		} else {
-			cf = nextColor()
+	var p *presenter
+	root, _, found := strings.Cut(name, " ")
+	if found {
+		parent := l.getPresenter(root)
+		p = &presenter{
+			colors: parent.colors,
+			name:   name,
+			prefix: parent.prefix,
+		}
+	} else {
+		cf := monochrome
+		if l.color {
+			if name == api.WatchLogger {
+				cf = makeColorFunc("92")
+			} else {
+				cf = nextColor()
+			}
+		}
+		p = &presenter{
+			colors: cf,
+			name:   name,
 		}
 	}
-	p := &presenter{
-		colors: cf,
-		name:   name,
-	}
 	l.presenters.Store(name, p)
+	l.computeWidth()
 	if l.prefix {
-		l.computeWidth()
 		l.presenters.Range(func(key, value interface{}) bool {
 			p := value.(*presenter)
 			p.setPrefix(l.width)
@@ -98,7 +109,7 @@ func (l *logConsumer) Log(container, message string) {
 	l.write(l.stdout, container, message)
 }
 
-// Log formats a log message as received from name/container
+// Err formats a log message as received from name/container
 func (l *logConsumer) Err(container, message string) {
 	l.write(l.stderr, container, message)
 }
@@ -115,9 +126,9 @@ func (l *logConsumer) write(w io.Writer, container, message string) {
 	timestamp := time.Now().Format(jsonmessage.RFC3339NanoFixed)
 	for _, line := range strings.Split(message, "\n") {
 		if l.timestamp {
-			fmt.Fprintf(w, "%s%s%s\n", p.prefix, timestamp, line)
+			_, _ = fmt.Fprintf(w, "%s%s%s\n", p.prefix, timestamp, line)
 		} else {
-			fmt.Fprintf(w, "%s%s\n", p.prefix, line)
+			_, _ = fmt.Fprintf(w, "%s%s\n", p.prefix, line)
 		}
 	}
 
