@@ -24,7 +24,6 @@ import (
 
 	"github.com/compose-spec/compose-go/v2/types"
 	"github.com/distribution/reference"
-	moby "github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/filters"
 	"github.com/docker/docker/api/types/image"
 	"github.com/docker/docker/client"
@@ -151,7 +150,7 @@ func (p *ImagePruner) namedImages(ctx context.Context) ([]string, error) {
 // The image name could either have been defined by the user or implicitly
 // created from the project + service name.
 func (p *ImagePruner) labeledLocalImages(ctx context.Context) ([]image.Summary, error) {
-	imageListOpts := moby.ImageListOptions{
+	imageListOpts := image.ListOptions{
 		Filters: filters.NewArgs(
 			projectFilter(p.project.Name),
 			// TODO(milas): we should really clean up the dangling images as
@@ -203,7 +202,6 @@ func (p *ImagePruner) filterImagesByExistence(ctx context.Context, imageNames []
 
 	eg, ctx := errgroup.WithContext(ctx)
 	for _, img := range imageNames {
-		img := img
 		eg.Go(func() error {
 			_, _, err := p.client.ImageInspectWithRaw(ctx, img)
 			if errdefs.IsNotFound(err) {
