@@ -28,7 +28,7 @@ import (
 	"github.com/compose-spec/compose-go/v2/types"
 	"github.com/distribution/reference"
 	"github.com/docker/buildx/driver"
-	moby "github.com/docker/docker/api/types"
+	"github.com/docker/docker/api/types/image"
 	"github.com/docker/docker/api/types/system"
 	"github.com/docker/docker/pkg/jsonmessage"
 	"github.com/docker/docker/registry"
@@ -62,6 +62,9 @@ func (s *composeService) push(ctx context.Context, project *types.Project, optio
 	w := progress.ContextWriter(ctx)
 	for _, service := range project.Services {
 		if service.Build == nil || service.Image == "" {
+			if options.ImageMandatory && service.Image == "" {
+				return fmt.Errorf("%q attribute is mandatory to push an image for service %q", "service.image", service.Name)
+			}
 			w.Event(progress.Event{
 				ID:     service.Name,
 				Status: progress.Done,
@@ -69,14 +72,12 @@ func (s *composeService) push(ctx context.Context, project *types.Project, optio
 			})
 			continue
 		}
-		service := service
 		tags := []string{service.Image}
 		if service.Build != nil {
 			tags = append(tags, service.Build.Tags...)
 		}
 
 		for _, tag := range tags {
-			tag := tag
 			eg.Go(func() error {
 				err := s.pushServiceImage(ctx, tag, info, s.configFile(), w, options.Quiet)
 				if err != nil {
@@ -117,7 +118,7 @@ func (s *composeService) pushServiceImage(ctx context.Context, tag string, info
 		return err
 	}
 
-	stream, err := s.apiClient().ImagePush(ctx, tag, moby.ImagePushOptions{
+	stream, err := s.apiClient().ImagePush(ctx, tag, image.PushOptions{
 		RegistryAuth: base64.URLEncoding.EncodeToString(buf),
 	})
 	if err != nil {
