@@ -25,7 +25,7 @@ import (
 	"time"
 
 	"github.com/compose-spec/compose-go/v2/types"
-	moby "github.com/docker/docker/api/types"
+	"github.com/docker/docker/api/types/container"
 )
 
 func (s *composeService) injectSecrets(ctx context.Context, project *types.Project, service types.ServiceConfig, id string) error {
@@ -50,7 +50,7 @@ func (s *composeService) injectSecrets(ctx context.Context, project *types.Proje
 			return err
 		}
 
-		err = s.apiClient().CopyToContainer(ctx, id, "/", &b, moby.CopyToContainerOptions{
+		err = s.apiClient().CopyToContainer(ctx, id, "/", &b, container.CopyToContainerOptions{
 			CopyUIDGID: config.UID != "" || config.GID != "",
 		})
 		if err != nil {
@@ -84,7 +84,7 @@ func (s *composeService) injectConfigs(ctx context.Context, project *types.Proje
 			return err
 		}
 
-		err = s.apiClient().CopyToContainer(ctx, id, "/", &b, moby.CopyToContainerOptions{
+		err = s.apiClient().CopyToContainer(ctx, id, "/", &b, container.CopyToContainerOptions{
 			CopyUIDGID: config.UID != "" || config.GID != "",
 		})
 		if err != nil {
